# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
# Copyright 2016 - Brocade Communications Systems, Inc.
#
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
import pkg_resources as pkg
import six
import sys

from mistralclient.api.v2 import executions
from mistralclient.commands.v2 import executions as execution_cmd
from mistralclient.tests.unit import base

EXEC = executions.Execution(
    mock,
    {
        'id': '123',
        'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
        'workflow_name': 'some',
        'workflow_namespace': '',
        'description': '',
        'state': 'RUNNING',
        'state_info': None,
        'created_at': '1',
        'updated_at': '1',
        'task_execution_id': None
    }
)

SUB_WF_EXEC = executions.Execution(
    mock,
    {
        'id': '456',
        'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
        'workflow_name': 'some_sub_wf',
        'workflow_namespace': '',
        'description': '',
        'state': 'RUNNING',
        'state_info': None,
        'created_at': '1',
        'updated_at': '1',
        'task_execution_id': 'abc'
    }
)

EX_RESULT = (
    '123',
    '123e4567-e89b-12d3-a456-426655440000',
    'some',
    '',
    '',
    '<none>',
    'RUNNING',
    None,
    '1',
    '1'
)

SUB_WF_EX_RESULT = (
    '456',
    '123e4567-e89b-12d3-a456-426655440000',
    'some_sub_wf',
    '',
    '',
    'abc',
    'RUNNING',
    None,
    '1',
    '1'
)


class TestCLIExecutionsV2(base.BaseCommandTest):

    stdout = six.moves.StringIO()
    stderr = six.moves.StringIO()

    def setUp(self):
        super(TestCLIExecutionsV2, self).setUp()

        # Redirect stdout and stderr so it doesn't pollute the test result.
        sys.stdout = self.stdout
        sys.stderr = self.stderr

    def tearDown(self):
        super(TestCLIExecutionsV2, self).tearDown()

        # Reset to original stdout and stderr.
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__

    def test_create_wf_input_string(self):
        self.client.executions.create.return_value = EXEC

        result = self.call(
            execution_cmd.Create,
            app_args=['id', '{ "context": true }']
        )

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_create_wf_input_file(self):
        self.client.executions.create.return_value = EXEC

        path = pkg.resource_filename(
            'mistralclient',
            'tests/unit/resources/ctx.json'
        )

        result = self.call(
            execution_cmd.Create,
            app_args=['id', path]
        )

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_create_with_description(self):
        self.client.executions.create.return_value = EXEC

        result = self.call(
            execution_cmd.Create,
            app_args=['id', '{ "context": true }', '-d', '']
        )

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_update_state(self):
        states = ['RUNNING', 'SUCCESS', 'PAUSED', 'ERROR', 'CANCELLED']

        for state in states:
            self.client.executions.update.return_value = executions.Execution(
                mock,
                {
                    'id': '123',
                    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
                    'workflow_name': 'some',
                    'workflow_namespace': '',
                    'description': '',
                    'state': state,
                    'state_info': None,
                    'created_at': '1',
                    'updated_at': '1',
                    'task_execution_id': None
                }
            )

            ex_result = list(EX_RESULT)
            ex_result[6] = state
            ex_result = tuple(ex_result)

            result = self.call(
                execution_cmd.Update,
                app_args=['id', '-s', state]
            )

            self.assertEqual(
                ex_result,
                result[1]
            )

    def test_update_invalid_state(self):
        states = ['IDLE', 'WAITING', 'DELAYED']

        for state in states:
            self.assertRaises(
                SystemExit,
                self.call,
                execution_cmd.Update,
                app_args=['id', '-s', state]
            )

    def test_resume_update_env(self):
        self.client.executions.update.return_value = EXEC

        result = self.call(
            execution_cmd.Update,
            app_args=['id', '-s', 'RUNNING', '--env', '{"k1": "foobar"}']
        )

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_update_description(self):
        self.client.executions.update.return_value = EXEC

        result = self.call(
            execution_cmd.Update,
            app_args=['id', '-d', 'foobar']
        )

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_list(self):
        self.client.executions.list.return_value = [EXEC, SUB_WF_EXEC]

        result = self.call(execution_cmd.List)

        self.assertEqual(
            [EX_RESULT, SUB_WF_EX_RESULT],
            result[1]
        )

    def test_list_with_pagination(self):
        self.client.executions.list.return_value = [EXEC]

        self.call(execution_cmd.List)
        self.client.executions.list.assert_called_once_with(
            limit=100,
            marker='',
            sort_dirs='asc',
            sort_keys='created_at',
            task=None
        )

        self.call(
            execution_cmd.List,
            app_args=[
                '--limit', '5',
                '--sort_dirs', 'id, Workflow',
                '--sort_keys', 'desc',
                '--marker', 'abc'
            ]
        )

        self.client.executions.list.assert_called_with(
            limit=5,
            marker='abc',
            sort_dirs='id, Workflow',
            sort_keys='desc',
            task=None
        )

    def test_get(self):
        self.client.executions.get.return_value = EXEC

        result = self.call(execution_cmd.Get, app_args=['id'])

        self.assertEqual(
            EX_RESULT,
            result[1]
        )

    def test_get_sub_wf_ex(self):
        self.client.executions.get.return_value = SUB_WF_EXEC

        result = self.call(execution_cmd.Get, app_args=['id'])

        self.assertEqual(
            SUB_WF_EX_RESULT,
            result[1]
        )

    def test_delete(self):
        self.call(execution_cmd.Delete, app_args=['id'])

        self.client.executions.delete.assert_called_once_with('id')

    def test_delete_with_multi_names(self):
        self.call(execution_cmd.Delete, app_args=['id1', 'id2'])

        self.assertEqual(2, self.client.executions.delete.call_count)
        self.assertEqual(
            [mock.call('id1'), mock.call('id2')],
            self.client.executions.delete.call_args_list
        )



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mistralclient.tests.unit.base_shell_test as base


class TestCLIBashCompletionV2(base.BaseShellTests):
    def test_bash_completion(self):
        bash_completion, stderr = self.shell('bash-completion')

        self.assertIn('bash-completion', bash_completion)
        self.assertFalse(stderr)



# Copyright 2014 - Mirantis, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import abc
import textwrap

from osc_lib.command import command
import six


DEFAULT_LIMIT = 100


@six.add_metaclass(abc.ABCMeta)
class MistralLister(command.Lister):
    @abc.abstractmethod
    def _get_format_function(self):
        raise NotImplementedError

    @abc.abstractmethod
    def _get_resources(self, parsed_args):
        """Gets a list of API resources (e.g. using client)."""
        raise NotImplementedError

    def _validate_parsed_args(self, parsed_args):
        # No-op by default.
        pass

    def take_action(self, parsed_args):
        self._validate_parsed_args(parsed_args)

        f = self._get_format_function()

        ret = self._get_resources(parsed_args)
        if not isinstance(ret, list):
            ret = [ret]

        data = [f(r)[1] for r in ret]

        if data:
            return f()[0], data
        else:
            return f()


def cut(string, length=25):
    if string and len(string) > length:
        return "%s..." % string[:length]
    else:
        return string


def wrap(string, width=25):
    if string and len(string) > width:
        return textwrap.fill(string, width)
    else:
        return string


def get_filters(parsed_args):
    filters = {}

    if parsed_args.filters:
        for f in parsed_args.filters:
            arr = f.split('=')

            if len(arr) != 2:
                raise ValueError('Invalid filter: %s' % f)

            filters[arr[0]] = arr[1]

    return filters



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock

from mistralclient.api.v2 import workbooks
from mistralclient.commands.v2 import workbooks as workbook_cmd
from mistralclient.tests.unit import base


WORKBOOK_DICT = {
    'name': 'a',
    'tags': ['a', 'b'],
    'created_at': '1',
    'updated_at': '1'
}


WB_DEF = """
---
version: '2.0

name: wb

workflows:
  wf1:
    tasks:
      task1:
        action: nova.servers_get server="1"
"""

WB_WITH_DEF_DICT = WORKBOOK_DICT.copy()
WB_WITH_DEF_DICT.update({'definition': WB_DEF})
WORKBOOK = workbooks.Workbook(mock, WORKBOOK_DICT)
WORKBOOK_WITH_DEF = workbooks.Workbook(mock, WB_WITH_DEF_DICT)


class TestCLIWorkbooksV2(base.BaseCommandTest):
    @mock.patch('argparse.open', create=True)
    def test_create(self, mock_open):
        self.client.workbooks.create.return_value = WORKBOOK

        result = self.call(workbook_cmd.Create, app_args=['wb.yaml'])

        self.assertEqual(('a', 'a, b', '1', '1'), result[1])

    @mock.patch('argparse.open', create=True)
    def test_update(self, mock_open):
        self.client.workbooks.update.return_value = WORKBOOK

        result = self.call(workbook_cmd.Update, app_args=['definition'])

        self.assertEqual(('a', 'a, b', '1', '1'), result[1])

    def test_list(self):
        self.client.workbooks.list.return_value = [WORKBOOK]

        result = self.call(workbook_cmd.List)

        self.assertEqual([('a', 'a, b', '1', '1')], result[1])

    def test_get(self):
        self.client.workbooks.get.return_value = WORKBOOK

        result = self.call(workbook_cmd.Get, app_args=['name'])

        self.assertEqual(('a', 'a, b', '1', '1'), result[1])

    def test_delete(self):
        self.call(workbook_cmd.Delete, app_args=['name'])

        self.client.workbooks.delete.assert_called_once_with('name')

    def test_delete_with_multi_names(self):
        self.call(workbook_cmd.Delete, app_args=['name1', 'name2'])

        self.assertEqual(2, self.client.workbooks.delete.call_count)
        self.assertEqual(
            [mock.call('name1'), mock.call('name2')],
            self.client.workbooks.delete.call_args_list
        )

    def test_get_definition(self):
        self.client.workbooks.get.return_value = WORKBOOK_WITH_DEF

        self.call(workbook_cmd.GetDefinition, app_args=['name'])

        self.app.stdout.write.assert_called_with(WB_DEF)

    @mock.patch('argparse.open', create=True)
    def test_validate(self, mock_open):
        self.client.workbooks.validate.return_value = {'valid': True}

        result = self.call(workbook_cmd.Validate, app_args=['wb.yaml'])

        self.assertEqual((True, None), result[1])

    @mock.patch('argparse.open', create=True)
    def test_validate_failed(self, mock_open):
        self.client.workbooks.validate.return_value = {
            'valid': False,
            'error': 'Invalid DSL...'
        }

        result = self.call(workbook_cmd.Validate, app_args=['wb.yaml'])

        self.assertEqual((False, 'Invalid DSL...'), result[1])



# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json

import six

from mistralclient.api import base
from mistralclient import utils


class Environment(base.Resource):
    resource_name = 'Environment'

    def _set_attributes(self):
        """Override loading of the "variables" attribute from text to dict."""
        for k, v in self._data.items():
            if k == 'variables' and isinstance(v, six.string_types):
                v = json.loads(v)

            try:
                setattr(self, k, v)
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass


class EnvironmentManager(base.ResourceManager):
    resource_class = Environment

    def create(self, **kwargs):
        # Check to see if the file name or URI is being passed in. If so,
        # read it's contents first.
        if 'file' in kwargs:
            file = kwargs['file']
            kwargs = utils.load_content(utils.get_contents_if_file(file))

        self._ensure_not_empty(name=kwargs.get('name', None),
                               variables=kwargs.get('variables', None))

        # Convert dict to text for the variables attribute.
        if isinstance(kwargs['variables'], dict):
            kwargs['variables'] = json.dumps(kwargs['variables'])

        return self._create('/environments', kwargs)

    def update(self, **kwargs):
        # Check to see if the file name or URI is being passed in. If so,
        # read it's contents first.
        if 'file' in kwargs:
            file = kwargs['file']
            kwargs = utils.load_content(utils.get_contents_if_file(file))

        name = kwargs.get('name', None)
        self._ensure_not_empty(name=name)

        # Convert dict to text for the variables attribute.
        if kwargs.get('variables') and isinstance(kwargs['variables'], dict):
            kwargs['variables'] = json.dumps(kwargs['variables'])

        return self._update('/environments', kwargs)

    def list(self):
        return self._list('/environments', response_key='environments')

    def get(self, name):
        self._ensure_not_empty(name=name)

        return self._get('/environments/%s' % name)

    def delete(self, name):
        self._ensure_not_empty(name=name)

        self._delete('/environments/%s' % name)



# Copyright 2017, OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json

from mistralclient.api import base


class EventTrigger(base.Resource):
    resource_name = 'EventTrigger'


class EventTriggerManager(base.ResourceManager):
    resource_class = EventTrigger

    def create(self, name, workflow_id, exchange, topic, event,
               workflow_input=None, workflow_params=None):
        self._ensure_not_empty(
            name=name,
            workflow_id=workflow_id
        )

        data = {
            'workflow_id': workflow_id,
            'name': name,
            'exchange': exchange,
            'topic': topic,
            'event': event
        }

        if workflow_input:
            data.update({'workflow_input': json.dumps(workflow_input)})

        if workflow_params:
            data.update({'workflow_params': json.dumps(workflow_params)})

        return self._create('/event_triggers', data)

    def list(self):
        return self._list('/event_triggers', response_key='event_triggers')

    def get(self, id):
        self._ensure_not_empty(id=id)

        return self._get('/event_triggers/%s' % id)

    def delete(self, id):
        self._ensure_not_empty(id=id)

        self._delete('/event_triggers/%s' % id)



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import six

from mistralclient.api import base

urlparse = six.moves.urllib.parse


class Task(base.Resource):
    resource_name = 'Task'


class TaskManager(base.ResourceManager):
    resource_class = Task

    def list(self, workflow_execution_id=None, marker='', limit=None,
             sort_keys='', sort_dirs='', fields=[], **filters):
        url = '/tasks'

        if workflow_execution_id:
            url = '/executions/%s/tasks' % workflow_execution_id

        url += '%s'

        qparams = {}

        if marker:
            qparams['marker'] = marker

        if limit and limit > 0:
            qparams['limit'] = limit

        if sort_keys:
            qparams['sort_keys'] = sort_keys

        if sort_dirs:
            qparams['sort_dirs'] = sort_dirs

        if fields:
            qparams['fields'] = ",".join(fields)

        for name, val in filters.items():
            qparams[name] = val

        query_string = ("?%s" % urlparse.urlencode(list(qparams.items()))
                        if qparams else "")

        return self._list(url % query_string, response_key='tasks')

    def get(self, id):
        self._ensure_not_empty(id=id)

        return self._get('/tasks/%s' % id)

    def rerun(self, task_ex_id, reset=True, env=None):
        url = '/tasks/%s' % task_ex_id

        body = {
            'id': task_ex_id,
            'state': 'RUNNING',
            'reset': reset
        }

        if env:
            body['env'] = json.dumps(env)

        return self._update(url, body)



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.api import base
from mistralclient import utils


class Workbook(base.Resource):
    resource_name = 'Workbook'


class WorkbookManager(base.ResourceManager):
    resource_class = Workbook

    def create(self, definition):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = utils.get_contents_if_file(definition)

        resp = self.http_client.post(
            '/workbooks',
            definition,
            headers={'content-type': 'text/plain'}
        )

        if resp.status_code != 201:
            self._raise_api_exception(resp)

        return self.resource_class(self, base.extract_json(resp, None))

    def update(self, definition):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = utils.get_contents_if_file(definition)

        resp = self.http_client.put(
            '/workbooks',
            definition,
            headers={'content-type': 'text/plain'}
        )

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        return self.resource_class(self, base.extract_json(resp, None))

    def list(self):
        return self._list('/workbooks', response_key='workbooks')

    def get(self, name):
        self._ensure_not_empty(name=name)

        return self._get('/workbooks/%s' % name)

    def delete(self, name):
        self._ensure_not_empty(name=name)

        self._delete('/workbooks/%s' % name)

    def validate(self, definition):
        self._ensure_not_empty(definition=definition)

        # If the specified definition is actually a file, read in the
        # definition file
        definition = utils.get_contents_if_file(definition)

        resp = self.http_client.post(
            '/workbooks/validate',
            definition,
            headers={'content-type': 'text/plain'}
        )

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        return base.extract_json(resp, None)



# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import copy
import json

from keystoneauth1 import exceptions


class Resource(object):
    resource_name = 'Something'
    defaults = {}

    def __init__(self, manager, data):
        self.manager = manager
        self._data = data
        self._set_defaults()
        self._set_attributes()

    def _set_defaults(self):
        for k, v in self.defaults.items():
            if k not in self._data:
                self._data[k] = v

    def _set_attributes(self):
        for k, v in self._data.items():
            try:
                setattr(self, k, v)
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def to_dict(self):
        return copy.deepcopy(self._data)

    def __str__(self):
        vals = ", ".join(["%s='%s'" % (n, v)
                          for n, v in self._data.items()])
        return "%s [%s]" % (self.resource_name, vals)


def _check_items(obj, searches):
    try:
        return all(getattr(obj, attr) == value for (attr, value) in searches)
    except AttributeError:
        return False


def extract_json(response, response_key):
    if response_key is not None:
        return get_json(response)[response_key]
    else:
        return get_json(response)


class ResourceManager(object):
    resource_class = None

    def __init__(self, http_client):
        self.http_client = http_client

    def find(self, **kwargs):
        return [i for i in self.list() if _check_items(i, kwargs.items())]

    def _ensure_not_empty(self, **kwargs):
        for name, value in kwargs.items():
            if value is None or (isinstance(value, str) and len(value) == 0):
                raise APIException(
                    400,
                    '%s is missing field "%s"' %
                    (self.resource_class.__name__, name)
                )

    def _copy_if_defined(self, data, **kwargs):
        for name, value in kwargs.items():
            if value is not None:
                data[name] = value

    def _create(self, url, data, response_key=None, dump_json=True):
        if dump_json:
            data = json.dumps(data)

        try:
            resp = self.http_client.post(url, data)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 201:
            self._raise_api_exception(resp)

        return self.resource_class(self, extract_json(resp, response_key))

    def _update(self, url, data, response_key=None, dump_json=True):
        if dump_json:
            data = json.dumps(data)

        try:
            resp = self.http_client.put(url, data)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        return self.resource_class(self, extract_json(resp, response_key))

    def _list(self, url, response_key=None):
        try:
            resp = self.http_client.get(url)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 200:
            self._raise_api_exception(resp)

        return [self.resource_class(self, resource_data)
                for resource_data in extract_json(resp, response_key)]

    def _get(self, url, response_key=None):
        try:
            resp = self.http_client.get(url)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code == 200:
            return self.resource_class(self, extract_json(resp, response_key))
        else:
            self._raise_api_exception(resp)

    def _delete(self, url):
        try:
            resp = self.http_client.delete(url)
        except exceptions.HttpError as ex:
            self._raise_api_exception(ex.response)

        if resp.status_code != 204:
            self._raise_api_exception(resp)

    def _plurify_resource_name(self):
        return self.resource_class.resource_name + 's'

    def _raise_api_exception(self, resp):
        try:
            error_data = (resp.headers.get("Server-Error-Message", None) or
                          get_json(resp).get("faultstring"))
        except ValueError:
            error_data = resp.content
        raise APIException(error_code=resp.status_code,
                           error_message=error_data)


def get_json(response):
    """Gets JSON representation of response.

    This method provided backward compatibility with old versions
    of requests library.
    """
    json_field_or_function = getattr(response, 'json', None)

    if callable(json_field_or_function):
        return response.json()
    else:
        return json.loads(response.content)


class APIException(Exception):
    def __init__(self, error_code=None, error_message=None):
        super(APIException, self).__init__(error_message)
        self.error_code = error_code
        self.error_message = error_message



# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from six.moves import configparser
from tempest.lib.cli import base


CLI_DIR = os.environ.get(
    'OS_MISTRALCLIENT_EXEC_DIR',
    os.path.join(os.path.abspath('.'), '.tox/functional/bin')
)
_CREDS_FILE = 'functional_creds.conf'


def credentials(group='admin'):
    """Retrieves credentials to run functional tests.

    Credentials are either read from the environment or from a config file
    ('functional_creds.conf'). Environment variables override those from the
    config file.

    The 'functional_creds.conf' file is the clean and new way to use (by
    default tox 2.0 does not pass environment variables).
    """
    if group == 'admin':
        username = os.environ.get('OS_USERNAME')
        password = os.environ.get('OS_PASSWORD')
        tenant_name = os.environ.get('OS_TENANT_NAME')
        user_domain = os.environ.get('OS_USER_DOMAIN_NAME')
        project_domain = os.environ.get('OS_PROJECT_DOMAIN_NAME')
    else:
        username = os.environ.get('OS_ALT_USERNAME')
        password = os.environ.get('OS_ALT_PASSWORD')
        tenant_name = os.environ.get('OS_ALT_TENANT_NAME')
        user_domain = os.environ.get('OS_ALT_USER_DOMAIN_NAME')
        project_domain = os.environ.get('OS_ALT_PROJECT_DOMAIN_NAME')

    auth_url = os.environ.get('OS_AUTH_URL')

    config = configparser.RawConfigParser()
    if config.read(_CREDS_FILE):
        username = username or config.get(group, 'user')
        password = password or config.get(group, 'pass')
        tenant_name = tenant_name or config.get(group, 'tenant')
        auth_url = auth_url or config.get('auth', 'uri')
        user_domain = user_domain or config.get(group, 'user_domain')
        project_domain = project_domain or config.get(group, 'project_domain')

    # TODO(ddeja): Default value of OS_AUTH_URL is to provide url to v3 API.
    # Since tempest openstack client doesn't properly handle it, we switch
    # it back to v2. Once tempest openstack starts to use v3, this can be
    # deleted.
    # https://github.com/openstack/tempest/blob/master/tempest/lib/cli/base.py#L363
    return {
        'username': username,
        'password': password,
        'tenant_name': tenant_name,
        'auth_url': auth_url.replace('v3', 'v2.0')
    }


class MistralCLIAuth(base.ClientTestBase):

    _mistral_url = None

    def _get_admin_clients(self):
        creds = credentials()

        clients = base.CLIClient(
            username=creds['username'],
            password=creds['password'],
            tenant_name=creds['tenant_name'],
            project_name=creds['tenant_name'],
            uri=creds['auth_url'],
            cli_dir=CLI_DIR
        )

        return clients

    def _get_clients(self):
        return self._get_admin_clients()

    def mistral(self, action, flags='', params='', fail_ok=False):
        """Executes Mistral command."""
        mistral_url_op = "--os-mistral-url %s" % self._mistral_url

        if 'WITHOUT_AUTH' in os.environ:
            return base.execute(
                'mistral %s' % mistral_url_op,
                action,
                flags,
                params,
                fail_ok,
                merge_stderr=False,
                cli_dir=''
            )
        else:
            return self.clients.cmd_with_auth(
                'mistral %s' % mistral_url_op,
                action,
                flags,
                params,
                fail_ok
            )

    def get_project_id(self, project='admin'):
        project_name = credentials(project)['tenant_name']

        admin_clients = self._get_clients()

        # TODO(mfedosin): when bug #1719687 is closed we should provide
        # domain names in related parameters, not just as abstract flags
        flags = "--os-user-domain-name default " \
                "--os-project-domain-name default " \
                "--os-identity-api-version 3"
        projects = self.parser.listing(
            admin_clients.openstack(
                'project show',
                params=project_name,
                flags=flags
            )
        )

        return [o['Value'] for o in projects if o['Field'] == 'id'][0]


class MistralCLIAltAuth(base.ClientTestBase):

    _mistral_url = None

    def _get_alt_clients(self):
        creds = credentials('demo')

        clients = base.CLIClient(
            username=creds['username'],
            password=creds['password'],
            project_name=creds['tenant_name'],
            tenant_name=creds['tenant_name'],
            uri=creds['auth_url'],
            cli_dir=CLI_DIR
        )

        return clients

    def _get_clients(self):
        return self._get_alt_clients()

    def mistral_alt(self, action, flags='', params='', mode='alt_user'):
        """Executes Mistral command for alt_user from alt_tenant."""
        mistral_url_op = "--os-mistral-url %s" % self._mistral_url

        return self.clients.cmd_with_auth(
            'mistral %s' % mistral_url_op, action, flags, params)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pkg_resources as pkg
from six.moves.urllib import parse
from six.moves.urllib import request

from mistralclient.api import base as api_base
from mistralclient.api.v2 import actions
from mistralclient.tests.unit.v2 import base


ACTION_DEF = """
---
version: 2.0

my_action:
  base: std.echo
  base-input:
    output: 'Bye!'
  output:
    info: <% $.output %>
"""

INVALID_ACTION_DEF = """
---
version: 2.0

my_action:
  base: std.echo
  unexpected-property: 'this should fail'
  base-input:
    output: 'Bye!'
  output:
    info: <% $.output %>
"""

ACTION = {
    'id': '123',
    'name': 'my_action',
    'input': '',
    'definition': ACTION_DEF
}

URL_TEMPLATE = '/actions'
URL_TEMPLATE_SCOPE = '/actions?scope=private'
URL_TEMPLATE_NAME = '/actions/%s'
URL_TEMPLATE_VALIDATE = '/actions/validate'


class TestActionsV2(base.BaseClientV2Test):
    def test_create(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json={'actions': [ACTION]},
                                status_code=201)

        actions = self.actions.create(ACTION_DEF)

        self.assertIsNotNone(actions)
        self.assertEqual(ACTION_DEF, actions[0].definition)

        last_request = self.requests_mock.last_request
        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)

    def test_create_with_file(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json={'actions': [ACTION]},
                                status_code=201)

        # The contents of action_v2.yaml must be identical to ACTION_DEF
        path = pkg.resource_filename(
            'mistralclient',
            'tests/unit/resources/action_v2.yaml'
        )

        actions = self.actions.create(path)

        self.assertIsNotNone(actions)
        self.assertEqual(ACTION_DEF, actions[0].definition)

        last_request = self.requests_mock.last_request
        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)

    def test_update_with_id(self):
        self.requests_mock.put(self.TEST_URL + URL_TEMPLATE_NAME % 123,
                               json={'actions': [ACTION]})

        actions = self.actions.update(ACTION_DEF, id=123)

        self.assertIsNotNone(actions)
        self.assertEqual(ACTION_DEF, actions[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual('scope=private', last_request.query)
        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)

    def test_update(self):
        self.requests_mock.put(self.TEST_URL + URL_TEMPLATE,
                               json={'actions': [ACTION]})

        actions = self.actions.update(ACTION_DEF)

        self.assertIsNotNone(actions)
        self.assertEqual(ACTION_DEF, actions[0].definition)

        last_request = self.requests_mock.last_request

        self.assertEqual('scope=private', last_request.query)
        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)

    def test_update_with_file_uri(self):
        self.requests_mock.put(self.TEST_URL + URL_TEMPLATE,
                               json={'actions': [ACTION]})

        # The contents of action_v2.yaml must be identical to ACTION_DEF
        path = pkg.resource_filename(
            'mistralclient',
            'tests/unit/resources/action_v2.yaml'
        )

        # Convert the file path to file URI
        uri = parse.urljoin('file:', request.pathname2url(path))

        actions = self.actions.update(uri)

        self.assertIsNotNone(actions)
        self.assertEqual(ACTION_DEF, actions[0].definition)

        last_request = self.requests_mock.last_request
        self.assertEqual('scope=private', last_request.query)
        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)

    def test_list(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'actions': [ACTION]})

        action_list = self.actions.list()

        self.assertEqual(1, len(action_list))

        action = action_list[0]

        self.assertEqual(
            actions.Action(self.actions, ACTION).to_dict(),
            action.to_dict()
        )

    def test_list_with_pagination(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'actions': [ACTION],
                                     'next': '/actions?fake'})

        action_list = self.actions.list(
            limit=1,
            sort_keys='created_at',
            sort_dirs='asc'
        )

        self.assertEqual(1, len(action_list))

        last_request = self.requests_mock.last_request

        # The url param order is unpredictable.
        self.assertEqual(['1'], last_request.qs['limit'])
        self.assertEqual(['created_at'], last_request.qs['sort_keys'])
        self.assertEqual(['asc'], last_request.qs['sort_dirs'])

    def test_list_with_no_limit(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'actions': [ACTION]})

        action_list = self.actions.list(limit=-1)

        self.assertEqual(1, len(action_list))

        last_request = self.requests_mock.last_request

        self.assertNotIn('limit', last_request.qs)

    def test_get(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE_NAME % 'action',
                               json=ACTION)

        action = self.actions.get('action')

        self.assertIsNotNone(action)
        self.assertEqual(
            actions.Action(self.actions, ACTION).to_dict(),
            action.to_dict()
        )

    def test_delete(self):
        url = self.TEST_URL + URL_TEMPLATE_NAME % 'action'
        m = self.requests_mock.delete(url, status_code=204)

        self.actions.delete('action')

        self.assertEqual(1, m.call_count)

    def test_validate(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE_VALIDATE,
                                json={'valid': True})

        result = self.actions.validate(ACTION_DEF)

        self.assertIsNotNone(result)
        self.assertIn('valid', result)
        self.assertTrue(result['valid'])

        last_request = self.requests_mock.last_request
        self.assertEqual(ACTION_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_validate_with_file(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE_VALIDATE,
                                json={'valid': True})

        # The contents of action_v2.yaml must be identical to ACTION_DEF
        path = pkg.resource_filename(
            'mistralclient',
            'tests/unit/resources/action_v2.yaml'
        )

        result = self.actions.validate(path)

        self.assertIsNotNone(result)
        self.assertIn('valid', result)
        self.assertTrue(result['valid'])

        last_request = self.requests_mock.last_request
        self.assertEqual(ACTION_DEF, last_request.text)
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_validate_failed(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE_VALIDATE,
                                json={"valid": False,
                                      "error": "mocked error message"})

        result = self.actions.validate(INVALID_ACTION_DEF)

        self.assertIsNotNone(result)
        self.assertIn('valid', result)
        self.assertFalse(result['valid'])
        self.assertIn('error', result)
        self.assertIn("mocked error message", result['error'])

        last_request = self.requests_mock.last_request
        self.assertEqual('text/plain', last_request.headers['content-type'])

    def test_validate_api_failed(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE_VALIDATE,
                                status_code=500,
                                json={})

        self.assertRaises(
            api_base.APIException,
            self.actions.validate,
            ACTION_DEF
        )

        last_request = self.requests_mock.last_request

        self.assertEqual('text/plain', last_request.headers['content-type'])
        self.assertEqual(ACTION_DEF, last_request.text)



# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import os.path
import tempfile
import yaml

from mistralclient import utils
from oslotest import base


ENV_DICT = {'k1': 'abc', 'k2': 123, 'k3': True}
ENV_STR = json.dumps(ENV_DICT)
ENV_YAML = yaml.safe_dump(ENV_DICT, default_flow_style=False)


class UtilityTest(base.BaseTestCase):

    def test_load_empty(self):
        self.assertDictEqual(dict(), utils.load_content(None))
        self.assertDictEqual(dict(), utils.load_content(''))
        self.assertDictEqual(dict(), utils.load_content('{}'))
        self.assertListEqual(list(), utils.load_content('[]'))

    def test_load_json_content(self):
        self.assertDictEqual(ENV_DICT, utils.load_content(ENV_STR))

    def test_load_json_file(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(ENV_STR.encode('utf-8'))
            f.flush()
            file_path = os.path.abspath(f.name)

            self.assertDictEqual(ENV_DICT, utils.load_file(file_path))

    def test_load_yaml_content(self):
        self.assertDictEqual(ENV_DICT, utils.load_content(ENV_YAML))

    def test_load_yaml_file(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(ENV_YAML.encode('utf-8'))
            f.flush()
            file_path = os.path.abspath(f.name)

            self.assertDictEqual(ENV_DICT, utils.load_file(file_path))

    def test_load_json(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(ENV_STR.encode('utf-8'))
            f.flush()
            self.assertDictEqual(ENV_DICT, utils.load_json(f.name))

        self.assertDictEqual(ENV_DICT, utils.load_json(ENV_STR))



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock

from mistralclient.api.v2 import services
from mistralclient.commands.v2 import services as service_cmd
from mistralclient.tests.unit import base


SERVICE_DICT = {
    'name': 'service_name',
    'type': 'service_type',
}

SERVICE = services.Service(mock, SERVICE_DICT)


class TestCLIServicesV2(base.BaseCommandTest):
    def test_list(self):
        self.client.services.list.return_value = [SERVICE]
        expected = (SERVICE_DICT['name'], SERVICE_DICT['type'],)

        result = self.call(service_cmd.List)

        self.assertListEqual([expected], result[1])



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json

from mistralclient.api import base as api_base
from mistralclient.api.v2 import executions
from mistralclient.tests.unit.v2 import base

# TODO(everyone): Later we need additional tests verifying all the errors etc.

EXEC = {
    'id': "123",
    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
    'workflow_name': 'my_wf',
    'workflow_namespace': '',
    'description': '',
    'state': 'RUNNING',
    'input': {
        "person": {
            "first_name": "John",
            "last_name": "Doe"
        }
    }
}

SUB_WF_EXEC = {
    'id': "456",
    'workflow_id': '123e4567-e89b-12d3-a456-426655440000',
    'workflow_name': 'my_sub_wf',
    'workflow_namespace': '',
    'task_execution_id': "abc",
    'description': '',
    'state': 'RUNNING',
    'input': {
        "person": {
            "first_name": "John",
            "last_name": "Doe"
        }
    }
}

SOURCE_EXEC = EXEC
SOURCE_EXEC['source_execution_id'] = EXEC['workflow_id']
URL_TEMPLATE = '/executions'
URL_TEMPLATE_ID = '/executions/%s'


class TestExecutionsV2(base.BaseClientV2Test):
    def test_create(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json=EXEC,
                                status_code=201)

        body = {
            'workflow_name': EXEC['workflow_name'],
            'description': '',
            'input': json.dumps(EXEC['input'])
        }

        ex = self.executions.create(
            EXEC['workflow_name'],
            EXEC['workflow_namespace'],
            EXEC['input']
        )

        self.assertIsNotNone(ex)

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            ex.to_dict()
        )

        self.assertDictEqual(body, self.requests_mock.last_request.json())

    def test_create_with_workflow_id(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json=EXEC,
                                status_code=201)

        body = {
            'workflow_id': EXEC['workflow_id'],
            'description': '',
            'input': json.dumps(EXEC['input'])
        }

        ex = self.executions.create(
            EXEC['workflow_id'],
            workflow_input=EXEC['input']
        )

        self.assertIsNotNone(ex)

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            ex.to_dict()
        )

        self.assertDictEqual(body, self.requests_mock.last_request.json())

    def test_create_with_source_execution_id(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json=SOURCE_EXEC,
                                status_code=201)

        body = {
            'description': '',
            'source_execution_id': SOURCE_EXEC['source_execution_id']
        }

        ex = self.executions.create(
            source_execution_id=SOURCE_EXEC['source_execution_id']
        )

        self.assertIsNotNone(ex)

        self.assertDictEqual(
            executions.Execution(self.executions, SOURCE_EXEC).to_dict(),
            ex.to_dict()
        )

        self.assertDictEqual(body, self.requests_mock.last_request.json())

    def test_create_failure1(self):
        self.requests_mock.post(self.TEST_URL + URL_TEMPLATE,
                                json=EXEC,
                                status_code=201)
        self.assertRaises(api_base.APIException, self.executions.create, '')

    def test_update(self):
        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
        self.requests_mock.put(url, json=EXEC)
        body = {
            'state': EXEC['state']
        }

        ex = self.executions.update(EXEC['id'], EXEC['state'])

        self.assertIsNotNone(ex)

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            ex.to_dict()
        )

        self.assertDictEqual(body, self.requests_mock.last_request.json())

    def test_update_env(self):
        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
        self.requests_mock.put(url, json=EXEC)
        body = {
            'state': EXEC['state'],
            'params': {
                'env': {'k1': 'foobar'}
            }
        }

        ex = self.executions.update(
            EXEC['id'],
            EXEC['state'],
            env={'k1': 'foobar'}
        )

        self.assertIsNotNone(ex)

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            ex.to_dict()
        )

        self.assertDictEqual(body, self.requests_mock.last_request.json())

    def test_list(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'executions': [EXEC, SUB_WF_EXEC]})

        execution_list = self.executions.list()

        self.assertEqual(2, len(execution_list))

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            execution_list[0].to_dict()
        )

        self.assertDictEqual(
            executions.Execution(self.executions, SUB_WF_EXEC).to_dict(),
            execution_list[1].to_dict()
        )

    def test_list_with_pagination(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'executions': [EXEC],
                                     'next': '/executions?fake'})

        execution_list = self.executions.list(
            limit=1,
            sort_keys='created_at',
            sort_dirs='asc'
        )

        self.assertEqual(1, len(execution_list))

        last_request = self.requests_mock.last_request

        # The url param order is unpredictable.
        self.assertEqual(['1'], last_request.qs['limit'])
        self.assertEqual(['created_at'], last_request.qs['sort_keys'])
        self.assertEqual(['asc'], last_request.qs['sort_dirs'])

    def test_list_with_no_limit(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'executions': [EXEC]})

        execution_list = self.executions.list(limit=-1)

        self.assertEqual(1, len(execution_list))

        last_request = self.requests_mock.last_request

        self.assertNotIn('limit', last_request.qs)

    def test_get(self):
        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
        self.requests_mock.get(url, json=EXEC)

        ex = self.executions.get(EXEC['id'])

        self.assertDictEqual(
            executions.Execution(self.executions, EXEC).to_dict(),
            ex.to_dict()
        )

    def test_get_sub_wf_ex(self):
        url = self.TEST_URL + URL_TEMPLATE_ID % SUB_WF_EXEC['id']
        self.requests_mock.get(url, json=SUB_WF_EXEC)

        ex = self.executions.get(SUB_WF_EXEC['id'])

        self.assertDictEqual(
            executions.Execution(self.executions, SUB_WF_EXEC).to_dict(),
            ex.to_dict()
        )

    def test_delete(self):
        url = self.TEST_URL + URL_TEMPLATE_ID % EXEC['id']
        self.requests_mock.delete(url, status_code=204)

        self.executions.delete(EXEC['id'])



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock

import mistralclient.tests.unit.base_shell_test as base


class TestShell(base.BaseShellTests):

    def test_help(self):
        """Test that client is not created for help and bash complete"""
        for command in ('-h',
                        '--help',
                        'help',
                        'help workbook-list',
                        'bash-completion'):
            with mock.patch('mistralclient.api.client.client') as client_mock:
                self.shell(command)
                self.assertFalse(client_mock.called)

    @mock.patch('mistralclient.api.client.client')
    def test_command_no_mistral_url(self, client_mock):
        self.shell(
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('', params[1]['mistral_url'])

    @mock.patch('mistralclient.api.client.client')
    def test_command_with_mistral_url(self, client_mock):
        self.shell(
            '--os-mistral-url=http://localhost:8989/v2 workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('http://localhost:8989/v2',
                         params[1]['mistral_url'])

    @mock.patch('mistralclient.api.client.determine_client_version')
    def test_mistral_version(self, client_mock):
        self.shell(
            '--os-mistral-version=v1 workbook-list'
        )
        self.assertTrue(client_mock.called)
        mistral_version = client_mock.call_args
        self.assertEqual('v1', mistral_version[0][0])

    @mock.patch('mistralclient.api.client.determine_client_version')
    def test_no_mistral_version(self, client_mock):
        self.shell('workbook-list')
        self.assertTrue(client_mock.called)
        mistral_version = client_mock.call_args
        self.assertEqual('v2', mistral_version[0][0])

    @mock.patch('mistralclient.api.client.client')
    def test_service_type(self, client_mock):
        self.shell('--os-mistral-service-type=test workbook-list')
        self.assertTrue(client_mock.called)
        parmters = client_mock.call_args
        self.assertEqual('test', parmters[1]['service_type'])

    @mock.patch('mistralclient.api.client.client')
    def test_no_service_type(self, client_mock):
        self.shell('workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('workflowv2', params[1]['service_type'])

    @mock.patch('mistralclient.api.client.client')
    def test_endpoint_type(self, client_mock):
        self.shell('--os-mistral-endpoint-type=adminURL workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('adminURL', params[1]['endpoint_type'])

    @mock.patch('mistralclient.api.client.client')
    def test_no_endpoint_type(self, client_mock):
        self.shell('workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('publicURL', params[1]['endpoint_type'])

    @mock.patch('mistralclient.api.client.client')
    def test_auth_url(self, client_mock):
        self.shell(
            '--os-auth-url=https://127.0.0.1:35357/v3 '
            '--os-username=admin '
            '--os-password=1234 '
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('https://127.0.0.1:35357/v3', params[1]['auth_url'])

    @mock.patch('mistralclient.api.client.client')
    def test_no_auth_url(self, client_mock):
        self.shell('workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('', params[1]['auth_url'])

    @mock.patch('mistralclient.api.client.client')
    def test_default_auth_url_with_os_password(self, client_mock):
        self.shell('--os-username=admin --os-password=1234 workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('http://localhost:35357/v3', params[1]['auth_url'])

    @mock.patch('mistralclient.api.client.client')
    def test_default_auth_url_with_os_auth_token(self, client_mock):
        self.shell(
            '--os-auth-token=abcd1234 '
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('http://localhost:35357/v3', params[1]['auth_url'])

    @mock.patch('mistralclient.api.client.client')
    def test_profile(self, client_mock):
        self.shell('--profile=SECRET_HMAC_KEY workbook-list')
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('SECRET_HMAC_KEY', params[1]['profile'])

    @mock.patch('mistralclient.api.client.client')
    def test_region_name(self, client_mock):
        self.shell('--os-region-name=RegionOne workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args

        self.assertEqual('RegionOne', params[1]['region_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_tenant_id_and_tenant_name(self, client_mock):
        self.shell(
            '--os-tenant-id=123tenant --os-tenant-name=fake_tenant'
            ' workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('fake_tenant', params[1]['project_name'])
        self.assertEqual('123tenant', params[1]['project_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_project_id_and_project_name(self, client_mock):
        self.shell(
            '--os-project-name=fake_tenant --os-project-id=123tenant'
            ' workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args

        self.assertEqual('fake_tenant', params[1]['project_name'])
        self.assertEqual('123tenant', params[1]['project_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_project_domain_name(self, client_mock):
        self.shell('--os-project-domain-name=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args

        self.assertEqual('default', params[1]['project_domain_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_project_domain_id(self, client_mock):
        self.shell('--os-project-domain-id=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args

        self.assertEqual('default', params[1]['project_domain_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_user_domain_name(self, client_mock):
        self.shell('--os-user-domain-name=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['user_domain_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_user_domain_id(self, client_mock):
        self.shell('--os-user-domain-id=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['user_domain_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_user_name_and_password(self, client_mock):
        self.shell(
            '--os-target-username=admin'
            ' --os-target-password=secret_pass workbook-list')
        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('admin', params[1]['target_username'])
        self.assertEqual('secret_pass', params[1]['target_api_key'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_tenant_name_and_id(self, client_mock):
        self.shell(
            '--os-target-tenant-id=123fake'
            ' --os-target-tenant-name=fake_target workbook-list')
        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('123fake', params[1]['target_project_id'])
        self.assertEqual('fake_target', params[1]['target_project_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_user_domain_id(self, client_mock):
        self.shell('--os-target-user-domain-id=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['target_user_domain_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_user_domain_name(self, client_mock):
        self.shell('--os-target-user-domain-name=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['target_user_domain_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_project_domain_id(self, client_mock):
        self.shell('--os-target-project-domain-id=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['target_project_domain_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_target_project_domain_name(self, client_mock):
        self.shell('--os-target-project-domain-name=default workbook-list')

        self.assertTrue(client_mock.called)

        params = client_mock.call_args
        self.assertEqual('default', params[1]['target_project_domain_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_no_domains_keystone_v3(self, client_mock):
        self.shell(
            '--os-auth-url=https://127.0.0.1:35357/v3 '
            '--os-username=admin '
            '--os-password=1234 '
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('https://127.0.0.1:35357/v3', params[1]['auth_url'])
        # For keystone v3 'default' values are automatically substituted for
        # project_domain_id and user_domain_id, if nothing was provided
        self.assertEqual('default', params[1]['project_domain_id'])
        self.assertEqual('default', params[1]['user_domain_id'])
        self.assertEqual('default', params[1]['target_project_domain_id'])
        self.assertEqual('default', params[1]['target_user_domain_id'])

    @mock.patch('mistralclient.api.client.client')
    def test_with_domain_names_keystone_v3(self, client_mock):
        self.shell(
            '--os-auth-url=https://127.0.0.1:35357/v3 '
            '--os-username=admin '
            '--os-password=1234 '
            '--os-project-domain-name=fake_domain '
            '--os-user-domain-name=fake_domain '
            '--os-target-project-domain-name=fake_domain '
            '--os-target-user-domain-name=fake_domain '
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('https://127.0.0.1:35357/v3', params[1]['auth_url'])
        # No need to substitute values for project_domain_id and
        # user_domain_id if related domain names were provided
        self.assertEqual('', params[1]['project_domain_id'])
        self.assertEqual('', params[1]['user_domain_id'])
        self.assertEqual('fake_domain', params[1]['project_domain_name'])
        self.assertEqual('fake_domain', params[1]['user_domain_name'])
        self.assertEqual(
            'fake_domain',
            params[1]['target_project_domain_name']
        )
        self.assertEqual('fake_domain', params[1]['target_user_domain_name'])

    @mock.patch('mistralclient.api.client.client')
    def test_no_domains_keystone_v2(self, client_mock):
        self.shell(
            '--os-auth-url=https://127.0.0.1:35357/v2.0 '
            '--os-username=admin '
            '--os-password=1234 '
            'workbook-list'
        )
        self.assertTrue(client_mock.called)
        params = client_mock.call_args
        self.assertEqual('https://127.0.0.1:35357/v2.0', params[1]['auth_url'])
        # For keystone v2 nothing is substituted
        self.assertEqual('', params[1]['project_domain_id'])
        self.assertEqual('', params[1]['user_domain_id'])
        self.assertEqual('', params[1]['target_project_domain_id'])
        self.assertEqual('', params[1]['target_user_domain_id'])



# Copyright 2014 - Mirantis, Inc.
# Copyright 2015 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json

from oslo_utils import uuidutils
import six

from mistralclient.api import base


urlparse = six.moves.urllib.parse


class Execution(base.Resource):
    resource_name = 'Execution'


class ExecutionManager(base.ResourceManager):
    resource_class = Execution

    def create(self, workflow_identifier='', namespace='',
               workflow_input=None, description='', source_execution_id=None,
               **params):
        ident = workflow_identifier or source_execution_id
        self._ensure_not_empty(workflow_identifier=ident)

        data = {
            'description': description,
        }

        if uuidutils.is_uuid_like(source_execution_id):
            data.update({'source_execution_id': source_execution_id})

        if workflow_identifier:
            if uuidutils.is_uuid_like(workflow_identifier):
                data.update({'workflow_id': workflow_identifier})
            else:
                data.update({'workflow_name': workflow_identifier})

        if namespace:
            data.update({'workflow_namespace': namespace})

        if workflow_input:
            if isinstance(workflow_input, six.string_types):
                data.update({'input': workflow_input})
            else:
                data.update({'input': json.dumps(workflow_input)})

        if params:
            data.update({'params': json.dumps(params)})

        return self._create('/executions', data)

    def update(self, id, state, description=None, env=None):
        data = {}

        if state:
            data['state'] = state

        if description:
            data['description'] = description

        if env:
            data['params'] = {'env': env}

        return self._update('/executions/%s' % id, data)

    def list(self, task=None, marker='', limit=None, sort_keys='',
             sort_dirs='', **filters):
        qparams = {}

        if task:
            qparams['task_execution_id'] = task

        if marker:
            qparams['marker'] = marker

        if limit and limit > 0:
            qparams['limit'] = limit

        if sort_keys:
            qparams['sort_keys'] = sort_keys

        if sort_dirs:
            qparams['sort_dirs'] = sort_dirs

        for name, val in filters.items():
            qparams[name] = val

        query_string = ("?%s" % urlparse.urlencode(list(qparams.items()))
                        if qparams else "")

        return self._list(
            '/executions%s' % query_string,
            response_key='executions',
        )

    def get(self, id):
        self._ensure_not_empty(id=id)

        return self._get('/executions/%s' % id)

    def delete(self, id):
        self._ensure_not_empty(id=id)

        self._delete('/executions/%s' % id)



# Copyright 2015 Huawei Technologies Co., Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistralclient.api.v2 import services
from mistralclient.tests.unit.v2 import base


SERVICE = {
    'name': 'service_name',
    'type': 'service_type',
}

URL_TEMPLATE = '/services'


class TestServicesV2(base.BaseClientV2Test):
    def test_list(self):
        self.requests_mock.get(self.TEST_URL + URL_TEMPLATE,
                               json={'services': [SERVICE]})

        service_list = self.services.list()

        self.assertEqual(1, len(service_list))

        srv = service_list[0]

        self.assertDictEqual(
            services.Service(self.services, SERVICE).to_dict(),
            srv.to_dict()
        )



# Copyright 2014 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import json
import six

from mistralclient.api import base

urlparse = six.moves.urllib.parse


class ActionExecution(base.Resource):
    resource_name = 'ActionExecution'


class ActionExecutionManager(base.ResourceManager):
    resource_class = ActionExecution

    def create(self, name, input=None, **params):
        self._ensure_not_empty(name=name)

        data = {'name': name}

        if input:
            data['input'] = json.dumps(input)

        if params:
            data['params'] = json.dumps(params)

        resp = self.http_client.post(
            '/action_executions',
            json.dumps(data)
        )

        if resp.status_code != 201:
            self._raise_api_exception(resp)

        return self.resource_class(self, base.get_json(resp))

    def update(self, id, state=None, output=None):
        self._ensure_not_empty(id=id)

        if not (state or output):
            raise base.APIException(
                400,
                "Please provide either state or output for action execution."
            )

        data = {}
        if state:
            data['state'] = state

        if output:
            data['output'] = output

        return self._update('/action_executions/%s' % id, data)

    def list(self, task_execution_id=None, limit=None):
        url = '/action_executions'

        if task_execution_id:
            url = '/tasks/%s/action_executions' % task_execution_id

        url += "%s"

        qparams = {}

        if limit and limit > 0:
            qparams['limit'] = limit

        query_string = ("?%s" % urlparse.urlencode(list(qparams.items()))
                        if qparams else "")

        return self._list(url % query_string, response_key='action_executions')

    def get(self, id):
        self._ensure_not_empty(id=id)

        return self._get('/action_executions/%s' % id)

    def delete(self, id):
        self._ensure_not_empty(id=id)

        self._delete('/action_executions/%s' % id)



# Copyright 2014 - Mirantis, Inc.
# Copyright 2016 - Brocade Communications Systems, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import json
import logging

from osc_lib.command import command

from mistralclient.commands.v2 import base
from mistralclient import utils

LOG = logging.getLogger(__name__)


def format_list(action_ex=None):
    columns = (
        'ID',
        'Name',
        'Workflow name',
        'Workflow namespace',
        'Task name',
        'Task ID',
        'State',
        'Accepted',
        'Created at',
        'Updated at'
    )

    if action_ex:
        data = (
            action_ex.id,
            action_ex.name,
            action_ex.workflow_name,
            action_ex.workflow_namespace,
            action_ex.task_name if hasattr(action_ex, 'task_name') else None,
            action_ex.task_execution_id,
            action_ex.state,
            action_ex.accepted,
            action_ex.created_at,
            action_ex.updated_at or '<none>'
        )
    else:
        data = (tuple('' for _ in range(len(columns))),)

    return columns, data


def format(action_ex=None):
    columns = (
        'ID',
        'Name',
        'Workflow name',
        'Workflow namespace',
        'Task name',
        'Task ID',
        'State',
        'State info',
        'Accepted',
        'Created at',
        'Updated at',
    )

    if action_ex:
        data = (
            action_ex.id,
            action_ex.name,
            action_ex.workflow_name,
            action_ex.workflow_namespace,
            action_ex.task_name if hasattr(action_ex, 'task_name') else None,
            action_ex.task_execution_id,
            action_ex.state,
            action_ex.state_info,
            action_ex.accepted,
            action_ex.created_at,
            action_ex.updated_at or '<none>'
        )
    else:
        data = (tuple('' for _ in range(len(columns))),)

    return columns, data


class Create(command.ShowOne):
    """Create new Action execution or just run specific action."""

    def produce_output(self, parsed_args, column_names, data):
        if not column_names:
            return 0

        return super(Create, self).produce_output(
            parsed_args,
            column_names,
            data
        )

    def get_parser(self, prog_name):
        parser = super(Create, self).get_parser(prog_name)

        parser.add_argument(
            'name',
            help='Action name to execute.'
        )
        parser.add_argument(
            dest='input',
            nargs='?',
            help='Action input.'
        )
        parser.add_argument(
            '-s',
            '--save-result',
            dest='save_result',
            action='store_true',
            help='Save the result into DB.'
        )
        parser.add_argument(
            '--run-sync',
            dest='run_sync',
            action='store_true',
            help='Run the action synchronously.'
        )
        parser.add_argument(
            '-t',
            '--target',
            dest='target',
            help='Action will be executed on <target> executor.'
        )

        return parser

    def take_action(self, parsed_args):
        params = {}

        if parsed_args.save_result:
            params['save_result'] = parsed_args.save_result

        if parsed_args.run_sync:
            params['run_sync'] = parsed_args.run_sync

        if parsed_args.target:
            params['target'] = parsed_args.target

        action_input = None

        if parsed_args.input:
            action_input = utils.load_json(parsed_args.input)

        mistral_client = self.app.client_manager.workflow_engine

        action_ex = mistral_client.action_executions.create(
            parsed_args.name,
            action_input,
            **params
        )

        if not parsed_args.run_sync and parsed_args.save_result:
            return format(action_ex)
        else:
            self.app.stdout.write("%s\n" % action_ex.output)

            return None, None


class List(base.MistralLister):
    """List all Action executions."""

    def _get_format_function(self):
        return format_list

    def get_parser(self, prog_name):
        parser = super(List, self).get_parser(prog_name)

        parser.add_argument(
            'task_execution_id',
            nargs='?',
            help='Task execution ID.'
        )
        parser.add_argument(
            '--limit',
            type=int,
            help='Maximum number of action-executions to return in a single '
                 'result. limit is set to %s by default. Use --limit -1 to '
                 'fetch the full result set.' % base.DEFAULT_LIMIT,
            nargs='?'
        )

        return parser

    def _get_resources(self, parsed_args):
        if parsed_args.limit is None:
            parsed_args.limit = base.DEFAULT_LIMIT

            LOG.info(
                "limit is set to %s by default. Set "
                "the limit explicitly using \'--limit\', if required. "
                "Use \'--limit\' -1 to fetch the full result set.",
                base.DEFAULT_LIMIT
            )

        mistral_client = self.app.client_manager.workflow_engine

        return mistral_client.action_executions.list(
            parsed_args.task_execution_id,
            limit=parsed_args.limit,
        )


class Get(command.ShowOne):
    """Show specific Action execution."""

    def get_parser(self, prog_name):
        parser = super(Get, self).get_parser(prog_name)

        parser.add_argument('action_execution', help='Action execution ID.')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        execution = mistral_client.action_executions.get(
            parsed_args.action_execution
        )

        return format(execution)


class Update(command.ShowOne):
    """Update specific Action execution."""

    def get_parser(self, prog_name):
        parser = super(Update, self).get_parser(prog_name)

        parser.add_argument(
            'id',
            help='Action execution ID.')
        parser.add_argument(
            '--state',
            dest='state',
            choices=['IDLE', 'RUNNING', 'SUCCESS', 'ERROR', 'CANCELLED'],
            help='Action execution state')
        parser.add_argument(
            '--output',
            dest='output',
            help='Action execution output')

        return parser

    def take_action(self, parsed_args):
        output = None
        if parsed_args.output:
            output = utils.load_json(parsed_args.output)

        mistral_client = self.app.client_manager.workflow_engine
        execution = mistral_client.action_executions.update(
            parsed_args.id,
            parsed_args.state,
            output
        )

        return format(execution)


class GetOutput(command.Command):
    """Show Action execution output data."""

    def get_parser(self, prog_name):
        parser = super(GetOutput, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            help='Action execution ID.')

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        output = mistral_client.action_executions.get(parsed_args.id).output

        try:
            output = json.loads(output)
            output = json.dumps(output, indent=4) + "\n"
        except Exception:
            LOG.debug("Task result is not JSON.")

        self.app.stdout.write(output or "\n")


class GetInput(command.Command):
    """Show Action execution input data."""

    def get_parser(self, prog_name):
        parser = super(GetInput, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            help='Action execution ID.'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine
        result = mistral_client.action_executions.get(parsed_args.id).input

        try:
            result = json.loads(result)
            result = json.dumps(result, indent=4) + "\n"
        except Exception:
            LOG.debug("Task result is not JSON.")

        self.app.stdout.write(result or "\n")


class Delete(command.Command):
    """Delete action execution."""

    def get_parser(self, prog_name):
        parser = super(Delete, self).get_parser(prog_name)

        parser.add_argument(
            'action_execution',
            nargs='+',
            help='Id of action execution identifier(s).'
        )

        return parser

    def take_action(self, parsed_args):
        mistral_client = self.app.client_manager.workflow_engine

        utils.do_action_on_many(
            lambda s: mistral_client.action_executions.delete(s),
            parsed_args.action_execution,
            "Request to delete action execution %s has been accepted.",
            "Unable to delete the specified action execution(s)."
        )
